@@ -17,9 +17,11 @@
 #include <netdb.h>
 #include <pthread.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include "protocol_binary.h"
 #include "cache.h"
+#include "logger.h"
 
 #include "sasl_defs.h"
 
@@ -162,6 +164,7 @@ enum conn_states {
     conn_closing,    /**< closing this connection */
     conn_mwrite,     /**< writing out many items sequentially */
     conn_closed,     /**< connection is closed */
+    conn_watch,      /**< held by the logger thread as a watcher */
     conn_max_state   /**< Max state value (used for assertion) */
 };
 
@@ -209,7 +212,7 @@ enum pause_thread_types {
 #define NREAD_CAS 6
 
 enum store_item_type {
-    NOT_STORED=0, STORED, EXISTS, NOT_FOUND
+    NOT_STORED=0, STORED, EXISTS, NOT_FOUND, TOO_LARGE, NO_MEMORY
 };
 
 enum delta_result_type {
@@ -238,6 +241,7 @@ struct thread_stats {
     pthread_mutex_t   mutex;
     uint64_t          get_cmds;
     uint64_t          get_misses;
+    uint64_t          get_expired;
     uint64_t          touch_cmds;
     uint64_t          touch_misses;
     uint64_t          delete_misses;
@@ -258,11 +262,11 @@ struct thread_stats {
  */
 struct stats {
     pthread_mutex_t mutex;
-    unsigned int  curr_items;
-    unsigned int  total_items;
+    uint64_t      curr_items;
+    uint64_t      total_items;
     uint64_t      curr_bytes;
-    unsigned int  curr_conns;
-    unsigned int  total_conns;
+    uint64_t      curr_conns;
+    uint64_t      total_conns;
     uint64_t      rejected_conns;
     uint64_t      malloc_fails;
     unsigned int  reserved_fds;
@@ -272,6 +276,7 @@ struct stats {
     uint64_t      touch_cmds;
     uint64_t      get_hits;
     uint64_t      get_misses;
+    uint64_t      get_expired;
     uint64_t      touch_hits;
     uint64_t      touch_misses;
     uint64_t      evictions;
@@ -294,6 +299,10 @@ struct stats {
     bool          lru_crawler_running; /* crawl in progress */
     uint64_t      lru_maintainer_juggles; /* number of LRU bg pokes */
     uint64_t      time_in_listen_disabled_us;  /* elapsed time in microseconds while server unable to process new connections */
+    uint64_t      log_worker_dropped; /* logs dropped by worker threads */
+    uint64_t      log_worker_written; /* logs written by worker threads */
+    uint64_t      log_watcher_skipped; /* logs watchers missed */
+    uint64_t      log_watcher_sent; /* logs sent to watcher buffers */
     struct timeval maxconns_entered;  /* last time maxconns entered */
 };
 
@@ -414,6 +423,7 @@ typedef struct {
     struct thread_stats stats;  /* Stats generated by this thread */
     struct conn_queue *new_conn_queue; /* queue of new connections to handle */
     cache_t *suffix_cache;      /* suffix cache */
+    logger *l;                  /* logger buffer */
 } LIBEVENT_THREAD;
 
 typedef struct {
@@ -572,6 +582,7 @@ extern int daemonize(int nochdir, int noclose);
 void memcached_thread_init(int nthreads, struct event_base *main_base);
 int  dispatch_event_add(int thread, conn *c);
 void dispatch_conn_new(int sfd, enum conn_states init_state, int event_flags, int read_buffer_size, enum network_transport transport);
+void sidethread_conn_close(conn *c);
 
 /* Lock wrappers for cache functions that are called from main loop. */
 enum delta_result_type add_delta(conn *c, const char *key,
@@ -583,8 +594,8 @@ conn *conn_from_freelist(void);
 bool  conn_add_to_freelist(conn *c);
 int   is_listen_thread(void);
 item *item_alloc(char *key, size_t nkey, int flags, rel_time_t exptime, int nbytes);
-item *item_get(const char *key, const size_t nkey);
-item *item_touch(const char *key, const size_t nkey, uint32_t exptime);
+item *item_get(const char *key, const size_t nkey, conn *c);
+item *item_touch(const char *key, const size_t nkey, uint32_t exptime, conn *c);
 int   item_link(item *it);
 void  item_remove(item *it);
 int   item_replace(item *it, item *new_it, const uint32_t hv);
