@@ -371,6 +371,10 @@ static void *worker_libevent(void *arg) {
     /* Any per-thread setup can happen here; memcached_thread_init() will block until
      * all threads have finished initializing.
      */
+    me->l = logger_create();
+    if (me->l == NULL) {
+        abort();
+    }
 
     register_thread_initialized();
 
@@ -463,6 +467,48 @@ void dispatch_conn_new(int sfd, enum conn_states init_state, int event_flags,
     }
 }
 
+/*
+ * Re-dispatches a connection back to the original thread. Can be called from
+ * any side thread borrowing a connection.
+ * TODO: Look into this. too complicated?
+ */
+#ifdef BOGUS_DEFINE
+void redispatch_conn(conn *c) {
+    CQ_ITEM *item = cqi_new();
+    char buf[1];
+    if (item == NULL) {
+        /* Can't cleanly redispatch connection. close it forcefully. */
+        /* FIXME: is conn_cleanup() necessary?
+         * if conn was handed off to a side thread it should be clean.
+         * could also put it into a "clean_me" state?
+         */
+        c->state = conn_closed;
+        close(c->sfd);
+        return;
+    }
+    LIBEVENT_THREAD *thread = c->thread;
+    item->sfd = sfd;
+    /* pass in the state somehow?
+    item->init_state = conn_closing; */
+    item->event_flags = c->event_flags;
+    item->conn = c;
+}
+#endif
+
+/* This misses the allow_new_conns flag :( */
+void sidethread_conn_close(conn *c) {
+    c->state = conn_closed;
+    if (settings.verbose > 1)
+        fprintf(stderr, "<%d connection closed from side thread.\n", c->sfd);
+    close(c->sfd);
+
+    STATS_LOCK();
+    stats.curr_conns--;
+    STATS_UNLOCK();
+
+    return;
+}
+
 /*
  * Returns true if this is the thread that listens for new TCP connections.
  */
@@ -486,22 +532,22 @@ item *item_alloc(char *key, size_t nkey, int flags, rel_time_t exptime, int nbyt
  * Returns an item if it hasn't been marked as expired,
  * lazy-expiring as needed.
  */
-item *item_get(const char *key, const size_t nkey) {
+item *item_get(const char *key, const size_t nkey, conn *c) {
     item *it;
     uint32_t hv;
     hv = hash(key, nkey);
     item_lock(hv);
-    it = do_item_get(key, nkey, hv);
+    it = do_item_get(key, nkey, hv, c);
     item_unlock(hv);
     return it;
 }
 
-item *item_touch(const char *key, size_t nkey, uint32_t exptime) {
+item *item_touch(const char *key, size_t nkey, uint32_t exptime, conn *c) {
     item *it;
     uint32_t hv;
     hv = hash(key, nkey);
     item_lock(hv);
-    it = do_item_touch(key, nkey, exptime, hv);
+    it = do_item_touch(key, nkey, exptime, hv, c);
     item_unlock(hv);
     return it;
 }
@@ -613,6 +659,7 @@ void threadlocal_stats_reset(void) {
 
         threads[ii].stats.get_cmds = 0;
         threads[ii].stats.get_misses = 0;
+        threads[ii].stats.get_expired = 0;
         threads[ii].stats.touch_cmds = 0;
         threads[ii].stats.touch_misses = 0;
         threads[ii].stats.delete_misses = 0;
@@ -653,6 +700,7 @@ void threadlocal_stats_aggregate(struct thread_stats *stats) {
 
         stats->get_cmds += threads[ii].stats.get_cmds;
         stats->get_misses += threads[ii].stats.get_misses;
+        stats->get_expired += threads[ii].stats.get_expired;
         stats->touch_cmds += threads[ii].stats.touch_cmds;
         stats->touch_misses += threads[ii].stats.touch_misses;
         stats->delete_misses += threads[ii].stats.delete_misses;
